/* global webAppCore */
(function() {
    // Event Listeners
    window.addEventListener("load",startApp,false);
    
    // Global Hooks
    window.wacMenuOptionSelected = onMenuOptionSelected;

    let isPaused = false;
    let currentLocation = null;
    let curBearing = 0;
    let mapLoaded = false;
    let isFirstPing = true;

    let bearingHolder;
    let bearingReadout;

    let storedLat = "-1";
    let storedLon = "-1";

    function startApp() {
        webAppCore.startApp(initApp);
    }
    
    function initApp() {
        webAppCore.setTheme("#ff4719");
        webAppCore.setTitle(webAppCore.getString("app_name"));

        storedLat = webAppCore.getVar("saved-lat","-1");
        storedLon = webAppCore.getVar("saved-lon","-1");
        
        webAppCore.handlePauseFunction(onPause);
        webAppCore.handleResumeFunction(onResume);

        linkLayouts();
        
        setupMenuOptions();
        setupToolbarOptions();

        if(webAppCore.locationSupported()) {
            webAppCore.setLocationListener(onLocation);
        } else {
            webAppCore.showAlertWithTitle("Find My Car requires location capabilities.","Location Not Found");
            webAppCore.clearSplash();
        }

        gooseUpMap();
    }
    
    function onPause() {
        isPaused = true;
    }
    
    function onResume() {
        isPaused = false;
    }

    function linkLayouts() {

        const feature = document.getElementById("feature");
        webAppCore.loadFeature(feature);

        document.getElementById("settings").addEventListener("click",function() {
            webAppCore.toggleMenu();
        });

        bearingHolder = document.getElementById("bearingHolder");
        bearingReadout = document.getElementById("bearingReadout");

        document.getElementById("imgCenter").onclick = function() {
            if(mapLoaded && currentLocation) {
                webAppCore.setCirclePosition(currentLocation.lat,currentLocation.lon);
                webAppCore.centerMap(currentLocation.lat,currentLocation.lon,18);
            }
        };

        document.getElementById("imgDrop").onclick = function() {
            if(mapLoaded && currentLocation) {
                webAppCore.clearPins();
                webAppCore.dropMapPin(currentLocation.lat,currentLocation.lon);
                webAppCore.setCirclePosition(currentLocation.lat,currentLocation.lon);

                webAppCore.storeVar("saved-lat",currentLocation.lat.toString());
                webAppCore.storeVar("saved-lon",currentLocation.lon.toString());
            }
        };
    }
    
    function setupMenuOptions() {
        webAppCore.clearMenu();

        webAppCore.addMenuOption("Clear Map","wacMenuOptionSelected","delete",null,"clearmap","File");
    }
    
    function setupToolbarOptions() {
        webAppCore.clearToolbar();
    }
    
    function onMenuOptionSelected(name,tag) {
        if(tag == "clearmap") {
            webAppCore.clearPins();

            webAppCore.storeVar("saved-lat","-1");
            webAppCore.storeVar("saved-lon","-1");

            if(currentLocation) {
                webAppCore.setCirclePosition(currentLocation.lat,currentLocation.lon);
            }
        }
    }

    function onLocation(location) {
        if(!location || isPaused) {
            return;
        }

        let calcSpeed = 0;
        let curTime = new Date().getTime();

        if(currentLocation) {
            let lastLat = currentLocation.lat;
            let lastLon = currentLocation.lon;
            let lastTme = currentLocation.tme;

            let metDiff = getDistanceFromLatLonInM(lastLat,lastLon,location.lat,location.lon);

            let miliDif = curTime - lastTme;
            let secDiff = miliDif / 1000.0;

            calcSpeed = metDiff / secDiff;

            if(location.deg) {
                curBearing = location.deg;
            } else {
                curBearing = getBearing(lastLat,lastLon,location.lat,location.lon);
            }

            let dir = bearingToDirection(curBearing);

            if(bearingHolder.style.display != "block") {
                bearingHolder.style.display = "block";
            }

            if(bearingReadout.innerHTML != dir) {
                bearingReadout.innerHTML = dir;
            }
        }

        currentLocation = {
            alt: location.alt,
            deg: location.deg,
            lat: location.lat,
            lon: location.lon,
            mps: location.mps,
            tme: curTime
        };

        if(!currentLocation.mps) {
            currentLocation.mps = calcSpeed;
        }

        if(mapLoaded) {
            webAppCore.setCirclePosition(currentLocation.lat,currentLocation.lon);

            if(isFirstPing) {
                webAppCore.centerMap(currentLocation.lat,currentLocation.lon,17);
            }
            
            isFirstPing = false;
        }

        
    }

    function getDistanceFromLatLonInM(lat1,lon1,lat2,lon2) {
        let R = 6371; // Radius of the earth in km
        let dLat = deg2rad(lat2-lat1);  // deg2rad below
        let dLon = deg2rad(lon2-lon1); 
        let a = 
            Math.sin(dLat/2) * Math.sin(dLat/2) +
            Math.cos(deg2rad(lat1)) * Math.cos(deg2rad(lat2)) * 
            Math.sin(dLon/2) * Math.sin(dLon/2)
        ; 
        let c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a)); 
        let d = R * c; // Distance in km

        let met = d * 1000;

        return met;
    }

    function deg2rad(deg) {
        return deg * (Math.PI/180);
    }

    function getBearing(lat1,lon1,lat2,lon2) {
        let dLon = deg2rad(lon2-lon1);
        lat1 = deg2rad(lat1);
        lat2 = deg2rad(lat2);
        let y = Math.sin(dLon) * Math.cos(lat2);
        let x = Math.cos(lat1)*Math.sin(lat2) - Math.sin(lat1)*Math.cos(lat2)*Math.cos(dLon);
        let rad = Math.atan2(y, x);
        let brng = deg2rad(rad);
        return (brng + 360) % 360;
    }

    function bearingToDirection(bearing) {
        let theDirection = "N";

        if(bearing >= 337.5 || bearing < 22.5) {
            theDirection = "N";
        }

        if(bearing >= 22.5 && bearing < 67.5) {
            theDirection = "NE";
        }

        if(bearing >= 67.5 && bearing < 112.5) {
            theDirection = "E";
        }

        if(bearing >= 112.5 && bearing < 157.5) {
            theDirection = "SE";
        }

        if(bearing >= 157.5 && bearing < 202.5) {
            theDirection = "S";
        }

        if(bearing >= 202.5 && bearing < 247.5) {
            theDirection = "SW";
        }

        if(bearing >= 247.5 && bearing < 292.5) {
            theDirection = "W";
        }

        if(bearing >= 292.5 && bearing < 337.5) {
            theDirection = "NW";
        }

        return theDirection;
    }

    function gooseUpMap() {
        webAppCore.showSplash("Loading Map...");
        webAppCore.showMap(document.getElementById("mapHolder"),onMapLoaded);
    }

    function onMapLoaded() {
        mapLoaded = true;

        if(storedLat != "-1" && storedLon != "-1") {
            isFirstPing = false;
            webAppCore.clearPins();
            webAppCore.dropMapPin(parseFloat(storedLat),parseFloat(storedLon));
            webAppCore.centerMap(parseFloat(storedLat),parseFloat(storedLon),17);
        }

        webAppCore.clearSplash();
        webAppCore.getCurrentLocation(onLocation);
    }
})();